/*
  a basic player
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */


#include "guiPlayer4.h"



GuiPlayer::GuiPlayer() { 
  resize(200,120);

  guiTimerDevice = new GuiTimerDevice(this);
  guiTimerDevice->setGeometry( 10, 10, 50, 50 );
  guiTimerDevice->show();

  b = new QBoxLayout( this, QBoxLayout::LeftToRight, 5 );
  b->addWidget(guiTimerDevice , AlignLeft|AlignTop|AlignBottom );


  guiDecoder = new GuiDecoder(this);
  guiDecoder->setGeometry( 10, 10, 50, 50 );
  guiDecoder->show();

  b->addWidget(guiDecoder , AlignLeft|AlignTop|AlignBottom );
  
  guiInfoDevice = new GuiInfoDevice(this);
  guiInfoDevice->setGeometry( 10, 10, 180, 180 );
  guiInfoDevice->show();

  b->addWidget(guiInfoDevice , AlignLeft|AlignTop|AlignBottom );


  // ok now the "uninteresting parts"
  buildGui();


  // Now we bild the "core" device tree.

  multicastDevice = new MulticastDevice();
  audioDevice = new AudioDeviceLinux("/dev/dsp",true);
  multicastDevice->addListener(audioDevice);
  multicastDevice->addListener(guiTimerDevice->getDevice());
  multicastDevice->addListener(guiInfoDevice->getDevice());


  // You should make sure that all devices are initialized
  // Most of them does not need an "openDevice" call but
  // the audioDevice _does_
  // A common error is to not open the device and put
  // them in a yafDecoder. But the yafDecoder then blocks
  // because he does not get the preffered Buffersize
  // of the devices. Alwas make sure your devices are open!
  audioDevice->openDevice();

}


GuiPlayer::~GuiPlayer() {
  // Every device in the tree must be deleted seperatley:
  delete audioDevice;
  delete multicastDevice;
}

/*
  The yafDecoder cannot be handled like an ordinary
  device (on this level).
*/
void GuiPlayer::setGenericStreamProducer(GenericStreamProducer* aDecoder) {
  guiDecoder->setDevice(aDecoder);
  MetaPlayer::setGenericStreamProducer(aDecoder);
}



OutputDevice* GuiPlayer::getInternalDeviceTree() {
  // this method is used by the metaPlayer. It return
  // the root node of the devicetree.
  return multicastDevice;
}


void GuiPlayer::buildGui() {
  QWidget* w=this;



  // two vertical boxes in the horizontal one    
  QBoxLayout * radioBoxes = new QBoxLayout( QBoxLayout::Down );    
  b->addLayout( radioBoxes );    
  // fill the leftmost vertical box  
  b1 = new QRadioButton( w, "radio button 1" );  
  b1->setText( "Single delivery" );  
  b1->setMinimumSize( b1->sizeHint() );  
  b1->setMaximumSize( 500, b1->minimumSize().height() );  

  radioBoxes->addWidget( b1, AlignLeft|AlignTop );  
  b2 = new QRadioButton( w, "rb2" ); 
  b2->setText( "continuous with notify" );  
  b2->setMinimumSize( b2->sizeHint() ); 
  b2->setMaximumSize( 500, b2->minimumSize().height() ); 
  radioBoxes->addWidget( b2, AlignLeft|AlignTop );  
  b3 = new QRadioButton( w, "rb3" );  
  b3->setText( "continuous no notify" );
  b3->setMinimumSize( b3->sizeHint() ); 
  b3->setMaximumSize( 500, b3->minimumSize().height() );   
  radioBoxes->addWidget( b3, AlignLeft|AlignTop );  
  // since none of those will stretch, add some stretch at the bottom  
  radioBoxes->addStretch( 1 );  
  // insert all of the radio boxes into the button group, so they'll  
  // switch each other off   
  bg = new QButtonGroup(); 

  // In a yafDecoder it is relative safe to make a single
  // delivery. (But not in a device.)
  bg->insert( b1 ,_DELIVER_MODE_SINGLE);   
  bg->insert( b2 ,_DELIVER_MODE_WITH_NOTIFY);
  bg->insert( b3 ,_DELIVER_MODE_NO_NOTIFY); 

  connect(bg,SIGNAL(pressed(int)),guiDecoder,SLOT(setDeliverMode(int)));

  // add some optional spacing between the radio buttons and the slider   
  b->addStretch( 1 );

  char* infoTxt="
  This demo shows how you can control the stream
  on device level. You have three options. 

  1.  You can confirms each PCM frame (time consuming
      only necessary for detailed analysis). 
      This easilsy leads to -deadlocks- thus you should
      avoid using -single- delivery where possible.

  2.  You can say that you only want to have a notify
      by the device. You wont get vor each PCM frame
      a notify. But for many device this delivery
      method is sufficient. (like the timerDevice
      in this example)


  3.  The third option sends no notify. Prefereable
      for audioDevices (/dev/dsp).


  Note: These switches should _not_ be used
        to block the yafDecoder. You have high level Commands
        for pause/play in the yafDecoder class.
        These switches are _only_ to set for a 
        _single_ device how exact it should inform the gui. ";
                

                 

  QMultiLineEdit* info=new QMultiLineEdit(this);
  info->setText(infoTxt);
  b->addWidget( info, AlignLeft|AlignTop|AlignBottom );
  

  
  confirm = new QPushButton( "Confirm", this, "confirm" );
  confirm->setMinimumSize( confirm->sizeHint() );    
  confirm->setMaximumSize( confirm->minimumSize().width(), 500 );    
  b->addWidget( confirm, AlignLeft|AlignTop|AlignBottom );

  connect( confirm, SIGNAL(pressed()), 
	   guiDecoder, SLOT( clearData() ) );



  QPushButton *quit = new QPushButton( "Quit", this, "quit" );
  quit->setFont( QFont( "Times", 18, QFont::Bold ) ); 
  quit->setMinimumSize( quit->sizeHint() );    
  quit->setMaximumSize( quit->minimumSize().width(), 500 );    
  b->addWidget( quit, AlignLeft|AlignTop|AlignBottom );
  connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );

}
