/*
  a generic Server for deviceConfig entries
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#ifndef __DEVICECONFIGSERVER_H
#define __DEVICECONFIGSERVER_H


#include <producer/core/producer.h>
#include <producer/core/threadState.h>
#include <deviceConfig/deviceConfig.h>



/*
  What does this server do?
  <p>
  The idea behind this is to have a central instance which generates
  _valid_ deviceConfig classes.
  <p>
  A streamProducer is a device which creates a thread.
  This thread enters a server to get the new deviceConfig
  entries. If it has these entries these entries _must_ be the
  same until the thread re-enters the server.
  The server coordintes the current and valid deviceConfig entries
  and the entries for the next call. Thus the server makes the
  "transition" from the old to the new data.
  <p>
  If a thread enters the server the server forwards the thread
  to the deviceConfigProducer. 

*/

class DeviceConfigServer : public DeviceConfigProducer,ThreadState {
  
  ThreadState* client;
  DeviceConfig* config;
  Producer* producer;

 public:
  DeviceConfigServer();
  ~DeviceConfigServer();

  void setNotifyClient(ThreadState* client);
  void updateDeviceConfig(DeviceConfig* newConfig);
 
  void setProducerEnter(int allow);
  void enqueueThreadState(int state);

  void setProducer(Producer* producer);
  Producer* getProducer();
  DeviceConfig* getDeviceConfig();

};

#endif

