/*
  device forwards data in realtime (trys to do so)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#include <devices/realTimeAudioDevice.h>

// buffer allocation prefferedSize*Constants

#define _TOTAL_BUFFER_MULTIPLIER       16
#define _DEVICE_BUFFER_MULTIPLIER      4


static void *readloopThread(void *arg){
  ((RealTimeAudioDevice*)arg)->readloop();
  return NULL;
}   



RealTimeAudioDevice::RealTimeAudioDevice(AudioDevice* device) {
  pthread_mutex_init(&mut,NULL);

  this->device=device;
  isShutdown=true;
}

RealTimeAudioDevice::~RealTimeAudioDevice() {
  stop();
  if (ringBuffer != NULL) {
    delete ringBuffer;
  }
}


char* RealTimeAudioDevice::getNodeName() {
  return "RealTimeAudioDevice";
}




int RealTimeAudioDevice::openDevice() {
  int back;
  cout << "+++++++++++++++++++++++RealTimeAudioDevice::open"<<endl;
  back=device->openDevice();
  pSize=device->getPreferredBufferSize();
  totalBufferSize=pSize*_TOTAL_BUFFER_MULTIPLIER;
  minDeviceBuffer=pSize*_DEVICE_BUFFER_MULTIPLIER;

  ringBuffer = new RingBuffer(totalBufferSize,minDeviceBuffer);

  start();
  return back;
}


int RealTimeAudioDevice::closeDevice() {
  int back;
  back=device->closeDevice();
  stop();
  delete ringBuffer;
  ringBuffer=NULL;
  return back;
}




void RealTimeAudioDevice::start() {
  cout << "RealTimeAudioDevice::start mut -s"<<endl;
  pthread_mutex_lock(&mut);
  if (isShutdown == true) {
    isShutdown=false;
    pthread_mutex_unlock(&mut);
    pthread_create(&tr,NULL,readloopThread,this);
    return;
  }
  pthread_mutex_unlock(&mut);
  cout << "RealTimeAudioDevice::start -e"<<endl;
}

int RealTimeAudioDevice::getPreferredBufferSize() {
  return pSize;
}
  




void RealTimeAudioDevice::stop() {
  void* ret;
  cout << "RealTimeAudioDevice::stop -s"<<endl;
  pthread_mutex_lock(&mut);
  if (isShutdown == false) {
    isShutdown=true;
    pthread_mutex_unlock(&mut);
    ringBuffer->exitWaitForData();
    ringBuffer->exitWaitForSpace();
    cout << "vor join"<<endl;
    pthread_join(tr,&ret);
    return;
  }
  pthread_mutex_unlock(&mut);
  cout << "RealTimeAudioDevice::stop -e"<<endl;
}


void RealTimeAudioDevice::flush() {
  while (ringBuffer->waitForSpace(totalBufferSize) == false) {
    cout << "flushing RealTimeAudioDevice"<<endl;
  }
}


void RealTimeAudioDevice::readloop() {
  MemChunk* gmem=new MemChunk(0);
  int bytes;

  pthread_mutex_lock(&mut);
  while(isShutdown==false) {
    pthread_mutex_unlock(&mut);

    ringBuffer->getReadArea(gmem,pSize);
    if (gmem->getLen() == 0) {
      cout << "readloop goes sleeping:"<<ringBuffer->getFillgrade()<<endl;
      ringBuffer->waitForData(pSize);
      ringBuffer->getReadArea(gmem,pSize);
      cout << "readloop wakeup:"<<ringBuffer->getFillgrade()<<endl;
      cout << "readloop wakeup got:"<<gmem->getLen()<<endl;
    }    
    bytes=device->rawWrite(gmem->getPtr(),gmem->getLen());
    ringBuffer->forwardReadPtr(bytes);
    ringBuffer->forwardLockPtr(bytes);
    pthread_mutex_lock(&mut);
  }
  delete gmem;
}


void RealTimeAudioDevice::writeIn(NodeDevice* source,DeviceConfig* buf) {
  MemChunk gmem(0);
  MemChunk* infoMem;
  AudioBuffer* audioBuffer=buf->getAudioBuffer();
  infoMem=audioBuffer->getMemChunk();
  int canWrite=infoMem->getLen();
  int didWrite;
  int restWrite;


  while (device->needInit(buf)) {
    // if we have the total buffsize we can be sure
    // the read thread is waiting!
    if (ringBuffer->waitForSpace(totalBufferSize)) {
      device->setDeviceConfig(buf);
      break;
    }
  }
  didWrite=0;
  while (didWrite < canWrite) {
    restWrite=canWrite-didWrite;
    ringBuffer->getWriteArea(&gmem,restWrite);
    
    if (gmem.getLen() < restWrite) {
      ringBuffer->waitForSpace(restWrite);
      ringBuffer->getWriteArea(&gmem,restWrite);
    }
    if (gmem.getLen() > restWrite) {
      gmem.setLen(restWrite);
    }

    memcpy(gmem.getPtr(),infoMem->getPtr()+didWrite,gmem.getLen());
    ringBuffer->forwardWritePtr(gmem.getLen());
    didWrite=didWrite+gmem.getLen();
  }
}



